# $Id: /local/CPAN/Handel/trunk/t/lib/Handel/Checkout/Plugin/NotISAPlugin.pm 1916 2007-06-24T15:35:46.298350Z claco  $
package Handel::Checkout::Plugin::NotISAPlugin;
use strict;
use warnings;

sub new {
    return bless {}, shift;
};

1;
