# $Id: /local/CPAN/Handel/trunk/t/lib/Handel/TestComponents/DefaultValues.pm 1916 2007-06-24T15:35:46.298350Z claco  $
package Handel::TestComponents::DefaultValues;
use strict;
use warnings;
use base qw/Handel::Components::DefaultValues/;

1;
