# $Id: /local/CPAN/Handel/trunk/t/lib/Handel/Subclassing/Currency.pm 1916 2007-06-24T15:35:46.298350Z claco  $
package Handel::Subclassing::Currency;
use strict;
use warnings;
use base qw/Handel::Currency/;

1;
